#include<readParameters.hpp>
#include<armadillo>

using namespace std;

class Simulation{
    public:
        Simulation(
            string parameters_path);
        Simulation(
            string parameters_path, 
            bool friction_parallelized);
        void main();
    protected:
        void setupModelVectorsMatrices();
        void setupBlockMatrix();
        void setupBlockMatrixDerivative();
        void setupPadVector();
        void setupPadVectorDerivative();
        void setupSaveVectorsMatrices();

        void setupSliderSpeedMode();

        void setupConstants();

        double phi(double y);

        void setSpeedAfterThreshold();
        void updateSliderSpeed(
            int time_step
        );

        void function_u(
            const arma::mat& current_block_matrix, 
            const arma::rowvec& current_pad_vector, 
            arma::mat& current_block_matrix_derivative
        );

        void function_x(
            const arma::mat& current_block_matrix, 
            const arma::rowvec& current_pad_vector, 
            arma::rowvec& current_pad_vector_derivative
        );
        void function_x_odof(
            const arma::rowvec& current_pad_vector,
            arma::rowvec& current_pad_vector_derivative
        );
        double calculatePadSupportDamper(
            arma::rowvec current_pad_vector
        );
        double calculatePadFriction(
            arma::mat current_block_matrix, 
            arma::rowvec current_pad_vector
        );

        void midpointMethod(
            int time_step
        );

        void oneDegreeOfFreedomMidpointMethod(
            int time_step
        );

        void writeValuesToLoggers(
            const int& time_step
        );

        void printMidPointMethod(
            arma::mat block_matrix, 
            arma::rowvec pad_vector, 
            string explanation_text = "");
        void runMidpointMethod();

        void saveToCsv(
            string filename, 
            string result_path);

        void midpointMethodParallel(
            int timestep, 
            arma::mat& block_matrix_parallel, 
            arma::rowvec& pad_vector_parallel,
            arma::mat block_matrix_derivative_parallel, 
            arma::rowvec pad_vector_derivative_parallel);
        void setupAndRunFrictionInParallel(
            int time_step,
            double slider_speed);
        void calculateFrictionInParallel();
        void saveToCsvParallel(
            const arma::colvec& armadillo_vector,
            const string& result_path,
            const double& slider_speed
        );

        string input_parameters_path;

        Parameters input_parameters;

        arma::mat block_matrix;
        arma::mat block_matrix_derivative;

        arma::rowvec pad_vector;
        arma::rowvec pad_vector_derivative;

        int number_of_time_steps;
        int number_of_save_steps;
        int number_of_interval_save_step;

        double m_u;		// mass of block
        double k_p;			// Stationary spring between pad and blocks
        double m_c_crit;		// Critical damping coefficient
	    double c_p;			// Damper between pad and surface
	    double k_c;			// Neighboring spring

        int updateSliderSpeedDtInterval; //translate the interval for updating slider speed based on dt

        bool loggingSpecificBlocks = false;
        arma::uvec block_position_column_index = {0};
        arma::uvec block_velocity_column_index = {1};

        arma::mat block_position_to_file;
        arma::mat block_velocity_to_file;
        arma::colvec pad_position_to_file;
        arma::colvec pad_velocity_to_file;
        arma::colvec pad_friction_to_file;

        int index_pad_friction_to_file = 0;
        int index_to_file = 0;

        int friction_zero_counter = 0;
};

class DebugSimulation: public Simulation{
    public:
        DebugSimulation(string debug_parameters_path) : Simulation(debug_parameters_path){};
        void main();

    protected:
        void printMatrix(arma::mat matrix, bool arra_format);
        void printRowVector(arma::rowvec vector);

};