#include <iostream>
#include "readParameters.hpp"

// Test to put libraries in hpp file

// #include <stdio.h>  /* defines FILENAME_MAX */

// #include <unistd.h>
// #define GetCurrentDir getcwd

 

using namespace std;


Parameters::Parameters()
{

}

Parameters::Parameters(string inputPath){
    cout << inputPath << endl;
    filePath = inputPath;
    loadFile();
    assignParameters();
}

void Parameters::loadFile(){
    config = YAML::LoadFile("doc/yaml/config.yaml");
    //config_categories = config["config_parameters_path"];
    //config_parameteres = YAML::LoadFile(config["config_parameters_path"].as<string>());
    read_parameters = YAML::LoadFile(filePath);
    read_default_parameters = YAML::LoadFile(config["config_parameters_default_path"].as<string>());
}



void Parameters::assignParameters(){

    parameters = read_parameters["Parameters"];
    debug_parameters = read_parameters["Debug"];

    default_parameters = read_default_parameters["Parameters"];
    default_debug_parameters = read_default_parameters["Debug"];

    // global constants

	dt = readParameterDouble("dt");
	seed = readParameterInt("seed");
	num_events = readParameterInt("num_events");

	// local constants
	N = readParameterInt("N");
	max_time = readParameterDouble("max_time");
	slider_speed = readParameterDouble("slider_speed");
	increment = readParameterDouble("increment");
	interval = readParameterDouble("interval");

    file_name = readParameterString("file_name");

	progress_indicator = readParameterBool("progress_indicator");

    m_F0 = readParameterDouble("m_F0");
    m_alpha = readParameterDouble("m_alpha");
	m_sigma = readParameterDouble("m_sigma");
	m_mass_x = readParameterDouble("m_mass_x");
	m_scale_mass = readParameterDouble("m_scale_mass");
	m_zeta = readParameterDouble("m_zeta");
	m_k_P0 = readParameterDouble("m_k_P0");
	m_scale_P = readParameterDouble("m_scale_P");
    m_scale_C = readParameterDouble("m_scale_C");

    m_t = readParameterDouble("m_t");
	m_v0 = readParameterDouble("m_v0");
	m_u_min = readParameterDouble("m_u_min");

    loggedBlocks = readParameterIntList("blocks");	

    start_speed_continuous = readParameterDouble("start_speed_continuous");
    end_speed_continuous = readParameterDouble("end_speed_continuous");

    save_interval_dt = readParameterInt("save_interval_dt");
    threshold_speed = readParameterDouble("threshold_speed");

    debug_no_friction = readDebugParameterBool("debug_no_friction");
    debug_no_neighbor_springs = readDebugParameterBool("debug_no_neighbor_springs");
    debug_no_stationary_springs = readDebugParameterBool("debug_no_stationary_springs");
    debug_no_damper = readDebugParameterBool("debug_no_damper");
    debug_no_min_speed = readDebugParameterBool("debug_no_min_speed");
    debug_no_pad = readDebugParameterBool("debug_no_pad");
    debug_negative_intial_values = readDebugParameterBool("debug_negative_initial_values");
    debug_no_random_displacements = readDebugParameterBool("debug_no_random_displacements");
    debug_only_negative_initial = readDebugParameterBool("debug_only_negative_initial");
    debug_special_phi = readDebugParameterBool("debug_special_phi");
    debug_pad_as_block = readDebugParameterBool("debug_pad_as_block");
    debug_stop_slider = readDebugParameterBool("debug_stop_slider");
    debug_stick_blocks = readDebugParameterBool("debug_stick_blocks");
    debug_write_blocks = readDebugParameterBool("debug_write_blocks");
    debug_only_write_friction = readDebugParameterBool("debug_only_write_friction");
    debug_continuous_slider_speed = readDebugParameterBool("debug_continuous_slider_speed");
    debug_one_degree_freedom_mode = readDebugParameterBool("debug_one_degree_freedom_mode");

}


int Parameters::readParameterInt(string parameter_name){
    if(!parameters[parameter_name]){
        return(default_parameters[parameter_name].as<int>());
    }
    return parameters[parameter_name].as<int>();
}

double Parameters::readParameterDouble(string parameter_name){
    if(!parameters[parameter_name]){
        return(default_parameters[parameter_name].as<double>());
    }
    return parameters[parameter_name].as<double>();
}

string Parameters::readParameterString(string parameter_name){
    if(!parameters[parameter_name]){
        return(default_parameters[parameter_name].as<string>());
    }
    return parameters[parameter_name].as<string>();
}

bool Parameters::readParameterBool(string parameter_name){
    if(!parameters[parameter_name]){
        return(default_parameters[parameter_name].as<bool>());
    }
    return parameters[parameter_name].as<bool>();
}

arma::uvec Parameters::readParameterIntList(string parameter_name){
    if(!parameters[parameter_name]){
        return(yamlListToArmadilloVector(default_parameters[parameter_name]));
    }
    return(yamlListToArmadilloVector(parameters[parameter_name]));
}

arma::uvec Parameters::yamlListToArmadilloVector(YAML::Node yamlList){
    arma::uvec armaVector(yamlList.size());
    for(size_t i = 0; i < yamlList.size(); i++){
        armaVector[i] = yamlList[i].as<int>();
    }
    return armaVector;
}

// Used fir debugging purposes in simulation //

bool Parameters::readDebugParameterBool(string debug_parameter_name){
    if(!debug_parameters[debug_parameter_name]){
        return(default_debug_parameters[debug_parameter_name].as<bool>());
    }
    return debug_parameters[debug_parameter_name].as<bool>();
}

/////////////DEBUG:Code////////////////

// Used for debugging purposes //

// int DebugParameters::printCurrentDirectory(){
//     char cCurrentPath[FILENAME_MAX];

//     if (!GetCurrentDir(cCurrentPath, sizeof(cCurrentPath)))
//     {
//         return errno;
//     }
//     cout << "not current" << endl;

//     cCurrentPath[sizeof(cCurrentPath) - 1] = '\0'; /* not really required */

//     cout << "The current working directory is " << cCurrentPath << endl;
// }

void DebugParameters::printLoadedFile(){
    cout << read_parameters << endl;
}

// Used for debugging purposes //

void DebugParameters::debugTest(){
    //printCurrentDirectory();
    printLoadedFile();
    loggedBlocks.print();
}

